# ------------------------------------------------------------------------------
import time
from persistent.list import PersistentList
from appy.gen import *
<!imports!>

class ModelClass: pass
class PodTemplate(ModelClass):
    '''This class represents a POD template for this application.'''
<!podTemplateBody!>
class Flavour(ModelClass):
    '''This class represents the Appy class used for defining a flavour.'''
    folder=True
<!flavourBody!>
    def onEdit(self, created):
        if created:
            nbOfFlavours = len(self.tool.flavours)
            if nbOfFlavours != 1:
                self.number = nbOfFlavours
                self.o.registerPortalTypes()
        # Call the custom flavour "onEdit" method if it exists
        customFlavour = self.__class__.__bases__[1]
        if customFlavour != Flavour:
            # There is a custom flavour
            if customFlavour.__dict__.has_key('onEdit'):
                customFlavour.__dict__['onEdit'](self, created)
    def getAttributeName(self, attributeType, klass, attrName=None):
        '''Some names of Flavour attributes are not easy to guess. For example,
           the attribute that stores, for a given flavour, the POD templates
           for class A that is in package x.y is "flavour.podTemplatesForx_y_A".
           Other example: the attribute that stores the editable default value
           of field "f1" of class x.y.A is "flavour.defaultValueForx_y_A_f1".
           This method generates the attribute name based on p_attributeType,
           a p_klass from the application, and a p_attrName (given only if
           needed, for example if p_attributeType is "defaultValue").

           p_attributeType may be:
               "defaultValue" (attribute that stores the editable default value
                 for a given p_attrName of a given p_klass);
               "podTemplates" (attributes that stores the POD templates that
                 are defined for a given p_klass)
               "resultColumns" (attribute that stores the list of columns that
                 must be shown when displaying instances of the a given root
                 p_klass)
               "optionalFields" (attribute that stores the list of optional
                 attributes that are in use in the current flavour for the
                 given p_klass)'''
        fullClassName = '%s_%s' % (klass.__module__.replace('.', '_'),
                                   klass.__name__)
        res = '%sFor%s' % (attributeType, fullClassName)
        if attrName:
            res += '_%s' % attrName
        return res

class Tool(ModelClass):
    '''This class represents the tool for this application.'''
    folder=True
    def getInitiator(self):
        '''Retrieves the object that triggered the creation of the object
           being currently created (if any).'''
        res = None
        initiatorUid = self.session['initiator']
        if initiatorUid:
            res = self.o.uid_catalog(UID=initiatorUid)[0].getObject().\
                  _appy_getWrapper(force=True)
        return res
<!toolBody!>
class AbstractWrapper:
    def __init__(self, o):
        self.__dict__['o'] = o
    def __setattr__(self, name, v):
        exec "self.o.set%s%s(v)" % (name[0].upper(), name[1:])
    def __cmp__(self, other):
        if other:
            return cmp(self.o, other.o)
        else:
            return 1
    def get_tool(self):
        return self.o.<!toolInstanceName!>._appy_getWrapper(force=True)
    tool = property(get_tool)
    def get_session(self):
        return self.o.REQUEST.SESSION
    session = property(get_session)
    def get_typeName(self):
        return self.__class__.__bases__[-1].__name__
    typeName = property(get_typeName)
    def get_id(self):
        return self.o.id
    id = property(get_id)
    def get_state(self):
        return self.o.portal_workflow.getInfoFor(self.o, 'review_state')
    state = property(get_state)
    def get_stateLabel(self):
        return self.o.utranslate(self.o.getWorkflowLabel(), domain='<!applicationName!>')
    stateLabel = property(get_stateLabel)
    def get_klass(self):
        return self.__class__.__bases__[1]
    klass = property(get_klass)

    def link(self, fieldName, obj):
        '''This method links p_obj to this one through reference field
           p_fieldName.'''
        if isinstance(obj, AbstractWrapper):
            obj = obj.o
        postfix = 'et%s%s' % (fieldName[0].upper(), fieldName[1:])
        # Update the Archetypes reference field
        exec 'objs = self.o.g%s()' % postfix
        if not objs:
            objs = []
        elif type(objs) not in (list, tuple):
            objs = [objs]
        objs.append(obj)
        exec 'self.o.s%s(objs)' % postfix
        # Update the ordered list of references
        sortedRefField = '_appy_%s' % fieldName
        if not hasattr(self.o.aq_base, sortedRefField):
            exec 'self.o.%s = PersistentList()' % sortedRefField
        getattr(self.o, sortedRefField).append(obj.UID())

    def create(self, fieldName, **kwargs):
        '''This method allows to create an object and link it to the current
           one through reference field named p_fieldName.'''
        # Determine object id and portal type
        portalType = self.o._appy_getRefPortalType(fieldName)
        if kwargs.has_key('id'):
            objId = kwargs['id']
            del kwargs['id']
        else:
            objId = '%s.%f' % (fieldName, time.time())
        # Where must I create te object?
        if hasattr(self, 'folder') and self.folder:
            folder = self.o
        else:
            folder = self.o.getParentNode()
        # Create the object
        folder.invokeFactory(portalType, objId)
        ploneObj = getattr(folder, objId)
        appyObj = ploneObj._appy_getWrapper(force=True)
        # Set object attributes
        ploneObj._appy_manageSortedRefs()
        for attrName, attrValue in kwargs.iteritems():
            setterName = 'set%s%s' % (attrName[0].upper(), attrName[1:])
            if isinstance(attrValue, AbstractWrapper):
                try:
                    refAppyType = getattr(appyObj.__class__.__bases__[-1],
                                          attrName)
                    appyObj.link(attrName, attrValue.o)
                except AttributeError, ae:
                    pass
            else:
                getattr(ploneObj, setterName)(attrValue)
        # Link the object to this one
        self.link(fieldName, ploneObj)
        try:
            appyObj.onEdit(True) # Call custom initialization
        except AttributeError:
            pass
        self.o.reindexObject()
        ploneObj.reindexObject()
        return appyObj

    def translate(self, label, domain='<!applicationName!>'):
        return self.o.utranslate(label, domain=domain)

<!wrappers!>
# ------------------------------------------------------------------------------
